unit uXSDtool;
// -------------------------------------------------------------------
// XSDTool is a utility to create Class-definitions from an XSD file.
// it is based on JanXMLParser which can be found at
// http://www.jansfreeware.com
//   I had to slightly modify the parser because it had a memory leak.
//   the corrected version is part of this archive.
// I also used MRUFileList from Brad Stowers.
//   unfortunately hist delphi-free-stuff pages are no longer available.
//   the unit therefore is included here.
//
// XSDobj and XSDparser are mine and can be used freely for everybody.
// XMLTools is mine and can also be used freely.
// Mylib is a collection of usefull and not so useful routines, but
//   included in almost all my projects. See what you can do with it.
// -------------------------------------------------------------------
// (c) 2005 Thomas Kerkmann
// email: thkerkmann@t-online.de
// -------------------------------------------------------------------
interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  Menus, ExtCtrls, ComCtrls, StdCtrls, xpDialogs,
  JanXMLParser2, ImgList, ActnList, ToolWin, MRUFList;

type
  TForm1 = class(TForm)
    StatusBar1: TStatusBar;
    f: TMainMenu;
    File1: TMenuItem;
    Open1: TMenuItem;
    XOpenDialog1: TXOpenDialog;
    N1: TMenuItem;
    Exit1: TMenuItem;
    Edit1: TMenuItem;
    Fullexpand1: TMenuItem;
    Fullcollapse1: TMenuItem;
    Parse1: TMenuItem;
    test1: TMenuItem;
    debug1: TMenuItem;
    PageControl1: TPageControl;
    tsTree: TTabSheet;
    tree: TTreeView;
    tsXML: TTabSheet;
    Memo2: TMemo;
    tsCode: TTabSheet;
    Memo1: TMemo;
    ToolBar1: TToolBar;
    ToolButton1: TToolButton;
    ToolButton2: TToolButton;
    ToolButton3: TToolButton;
    ToolButton4: TToolButton;
    ToolButton5: TToolButton;
    ToolButton6: TToolButton;
    ToolButton7: TToolButton;
    ToolButton8: TToolButton;
    ActionList1: TActionList;
    acOpen: TAction;
    acExit: TAction;
    ImageList1: TImageList;
    acExpand: TAction;
    acCollapse: TAction;
    acCreateClass: TAction;
    acSave: TAction;
    ToolButton9: TToolButton;
    Options1: TMenuItem;
    Save1: TMenuItem;
    askoverride1: TMenuItem;
    promptaftersave1: TMenuItem;
    MRU: TMRUFileList;
    procedure FormCreate(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure debug1Click(Sender: TObject);
    procedure acOpenExecute(Sender: TObject);
    procedure acExitExecute(Sender: TObject);
    procedure acExpandExecute(Sender: TObject);
    procedure acCollapseExecute(Sender: TObject);
    procedure acCreateClassExecute(Sender: TObject);
    procedure PageControl1Change(Sender: TObject);
    procedure acSaveExecute(Sender: TObject);
    procedure askoverride1Click(Sender: TObject);
    procedure promptaftersave1Click(Sender: TObject);
    procedure Memo1Change(Sender: TObject);
    procedure MRUMRUItemClick(Sender: TObject; AFilename: string);
  private
    { Private-Deklarationen }
    dom: tJanXMLParser2;
    fn_xsd: string;
    fn_time: string;
    bChanged: boolean;
    procedure DoOpen(const aFilename: string);
    procedure SetActiveActions;
    procedure ParseTree;
    procedure ParseTreeNode(node: TTreeNode; dnode: TjanXMLNode2);
    procedure parsedef;
    procedure parseSchema(xSchema: tJanXMLNode2);
    procedure Logout(const s: string);
    procedure loadpos;
    procedure savepos;
    procedure loadreg;
    procedure savereg;
  public
    { Public-Deklarationen }
  end;

var
  Form1: TForm1;

implementation
uses
  Registry,
  uXSDobj,
  uXSDParser;

const
 RegKeyName = 'Software\XSDTool';

{$R *.DFM}

procedure TForm1.FormCreate(Sender: TObject);
begin
  loadreg;
  loadpos;
  Memo1.Clear;
  Memo2.Clear;
  dom := tJanXMLParser2.Create;
  bChanged := false;

  SetActiveActions;
end;

procedure TForm1.FormClose(Sender: TObject; var Action: TCloseAction);
begin
  dom.free;
  savepos;
end;

procedure TForm1.debug1Click(Sender: TObject);
begin
  debug1.checked := not debug1.checked;
  savereg;
end;

procedure TForm1.askoverride1Click(Sender: TObject);
begin
  askoverride1.checked := not askoverride1.checked;
  savereg;
end;

procedure TForm1.promptaftersave1Click(Sender: TObject);
begin
  promptaftersave1.checked := not promptaftersave1.checked;
  savereg;
end;

procedure TForm1.Logout(const s: string);
begin
  Memo1.Lines.Add(s);
end;

// ------------------------------------------------------------------
// read into treeview

procedure TForm1.ParseTreeNode(node: TTreeNode; dnode: TjanXMLNode2);
var
  n2: TTreenode;
  dn2: TjanXMLNode2;
  i, c: integer;
  nodename: string;
begin
  c := dnode.nodes.Count;
  if c = 0 then
    exit;
  for i := 0 to c - 1 do
  begin
    dn2 := TjanXMLNode2(dnode.Nodes[i]);
    nodename := dn2.name;
    n2 := tree.items.AddChildObject(node, nodename, dn2);
    parsetreenode(n2, dn2);
  end;
end;

procedure TForm1.ParseTree;
var
  n: TTreeNode;
  dn: TjanXMLNode2;
  nodename: string;
begin
  dn := dom;
  nodename := dn.name;
  n := tree.Items.AddObject(nil, nodename, dn);
  parsetreenode(n, dn);
end;

// ------------------------------------------------------------------
// XSD parsing

procedure TForm1.parseSchema(xSchema: tJanXMLNode2);
var
  Parser: tXSDParser;
  m: tMemoryStream;
begin // procedure parseSchema(dn:tJanXMLNode2)
  Parser := tXSDParser.Create;
  if debug1.checked then
    Parser.OnLogout := Logout;

  Parser.parseSchema(xSchema);

  Parser.ClassDefs.XSDTimestamp := fn_time;
  Parser.ClassDefs.XSDFilename := fn_xsd;

  // Parser.ClassDefs.CreateUnit(fn_pas, sa);
  m := tMemoryStream.Create;
  Parser.ClassDefs.SaveToStream(m);
  m.Position := 0;
  Memo1.Lines.LoadFromStream(m);
  m.free;
  Parser.Free;

  PageControl1.ActivePage := tsCode;
  SetActiveActions;
end; // procedure parseSchema(dn:tJanXMLNode2)

procedure TForm1.parsedef;
var
  i: integer;
begin
  Memo1.Clear;
  if dom.name = xsschema then
    parseSchema(dom)
  else
    messagedlg('No schema!', mtError, [mbOk], 0);
end;

// ------------------------------------------------------------------
// actions

procedure TForm1.acOpenExecute(Sender: TObject);
begin
  if XOpenDialog1.Execute then
    DoOpen(XOpenDialog1.Filename);
end;

procedure TForm1.MRUMRUItemClick(Sender: TObject; AFilename: string);
begin
  if FileExists(aFilename) then
    DoOpen(aFilename)
  else
    MessageDlg('File no longer available: ' + aFilename, mtError, [mbOk], 0);
end;

procedure TForm1.DoOpen(const aFilename: string);
begin
  fn_xsd := aFilename;
  MRU.InsertItem(0, fn_xsd);
  fn_Time := FormatDateTime('c', FileDateToDateTime(FileAge(fn_xsd)));

  tree.items.BeginUpdate;
  tree.items.clear;

  dom.LoadXML(fn_xsd);
  memo2.Text := dom.xml;
  memo1.Clear;
  bChanged := false;

  parsetree;

  tree.items.EndUpdate;
  tree.FullExpand;

  PageControl1.ActivePage := tsXML;
  SetActiveActions;
end;

procedure TForm1.acExitExecute(Sender: TObject);
begin
  close;
end;

procedure TForm1.acExpandExecute(Sender: TObject);
begin
  tree.FullExpand;
end;

procedure TForm1.acCollapseExecute(Sender: TObject);
begin
  tree.FullCollapse;
end;

procedure TForm1.acCreateClassExecute(Sender: TObject);
begin
  parsedef;
  // bchanged := true;
  PageControl1.ActivePage := tsCode;
  SetActiveActions;
end;

procedure TForm1.acSaveExecute(Sender: TObject);
var
  fn_pas: string;
  s: string;
begin
  // build destination file name
  fn_pas := ExtractFilePath(fn_xsd) + 'u' +
    ChangeFileExt(ExtractFileName(fn_xsd), '.pas');

  if FileExists(fn_pas) and askoverride1.checked then
    if MessageDlg(fn_pas + ' already exists. Do you want to override?',
      mtConfirmation, [mbYes, mbNo], 0) = mrNo then
      exit;
  Memo1.Lines.SaveToFile(fn_pas);
  bChanged := false;
  SetActiveActions;

  s := 'Delphi class definition was saved to ' + fn_pas;
  if promptaftersave1.checked then
    MessageDlg(s, mtInformation, [mbOk], 0);
  Statusbar1.SimpleText := s;
end;

procedure TForm1.PageControl1Change(Sender: TObject);
begin
  SetActiveActions;
end;

procedure TForm1.SetActiveActions;
begin
  acCreateClass.Enabled := (tree.Items.Count > 0);

  acExpand.Enabled := (Pagecontrol1.Activepage = tsTree)
    and (tree.Items.Count > 0);
  acCollapse.Enabled := (Pagecontrol1.Activepage = tsTree)
    and (tree.Items.Count > 0);

  acSave.Enabled := (PageControl1.ActivePage = tsCode)
    and (Memo1.Lines.Count > 0) and bChanged;
end;

// ------------------------------------------------------------------
// options

procedure TForm1.loadpos;
var
  reg: tRegInifile;
begin
  reg := tRegIniFile.Create(RegKeyName);
  with reg do
  begin
    left := ReadInteger('Position', 'left', left);
    top := ReadInteger('Position', 'top', top);
    width := ReadInteger('Position', 'width', width);
    height := ReadInteger('Position', 'height', height);
  end;
  reg.Free;
end;

procedure TForm1.savepos;
var
  reg: tRegInifile;
begin
  reg := tRegIniFile.Create(RegKeyName);
  with reg do
  begin
    WriteInteger('Position', 'left', left);
    WriteInteger('Position', 'top', top);
    WriteInteger('Position', 'width', width);
    WriteInteger('Position', 'height', height);
  end;
  reg.Free;
end;

procedure TForm1.loadreg;
var
  reg: tRegInifile;
begin
  reg := tRegIniFile.Create(RegKeyName);
  with reg do
  begin
    debug1.checked := ReadBool('Options', 'Debug', false);
    askoverride1.checked := Readbool('Options', 'AskBeforeOverride', true);
    promptaftersave1.checked := ReadBool('Options', 'PromptAfterSave', false);
  end;
  reg.Free;
end;

procedure TForm1.savereg;
var
  reg: tRegInifile;
begin
  reg := tRegIniFile.Create(RegKeyName);
  with reg do
  begin
    WriteBool('Options', 'Debug', debug1.checked);
    Writebool('Options', 'AskBeforeOverride', askoverride1.checked);
    WriteBool('Options', 'PromptAfterSave', promptaftersave1.checked);
  end;
  reg.Free;
end;

procedure TForm1.Memo1Change(Sender: TObject);
begin
  bChanged := true;
end;

end.

